%CDNS
% Numerical solution of nonstationary convection-diffusion equation: 
%	dv/dt + udv/dx - mu*d^2v/dx^2 = q,  
%	v(t,0)/dx = fL(t),  dv(t,1)/dx = fR(t)
%	q(t,x) = mu*beta^2*cos(beta(x-ut))
%	Exact solution: 
%	v = cos(beta(x-ut)) + exp(-alpha^2*mu*t)cos(alpha(x-ut))

% Copyright 2001 P. Wesseling
% This program and its subprograms may be freely used, modified and distributed
% under the GNU General Public License: http://www.gnu.org/copyleft/gpl.html

% Theory is given in Section 4.5 of
%	Computational Fluid Dynamics
%	Lecture Notes by P. Wesseling
%	Department of Applied Mathematical Analysis, ITS, TU Delft
% See http://ta.twi.tudelft.nl/nw/users/wesseling/

% This program generates Fig. 4.3 in the Lecture Notes

% Functions called: exact_solution, right_side,  fR

		% .......................Input..............................
mu = 0.00005;	% Diffusion coefficient
u = 1.1;		% Velocity coefficient
alpha = 4*pi;	% Coefficient in exact solution 
beta = 2*pi;	% Coefficient in exact solution 
omega = 1/2;	% Parameter of omega-scheme 
scheme = 2;	% Enter 1 for upwind scheme or something else for central scheme
inhom = 1;	% Enter 0 for homogeneous Neumann boundary condition at x=1
		% or 1 for inhomogeneous Neumann boundary condition
J = 31;		% Number of cells is J-1
T = 1;		% Final time
tau = 1/30;	% Time step
		% ....................End of input...........................

	% 	Vertex-centered uniform grid

	%                              
	%     x=0       J-1 cells     x=1
	%      |-------|-------|-------|
	%      1       2               J 

h = 1/(J-1);		% Mesh size
N = floor(T/tau);	% Number of time steps
x = linspace(0,1,J);	% Grid node locations: x_j = (j-1)/(J-1)
hh = h*ones(size(x')); 
hh(J,1) = h/2;		% Last cell has half size

	% Matrix generation
	% Coefficients in numerical flux are called beta:
	% F_{j+1/2} = beta0(j)v(j) + beta1(j+1)v(j+1), eq. (2.27)

e = ones(J,1); 
betadiff = (mu/h)*e;		% Contribution of diffusion		
if scheme == 1			% Upwind scheme
  beta0 = u*e + betadiff;
  beta1 = - betadiff;
else				% Central scheme
  beta0 = (u/2)*e + betadiff;
  beta1 = beta0 - 2*betadiff;
end
beta0(J) = u;			% Correction for Neumann boundary condition

L_h = spdiags([-beta0 beta0-beta1 beta1], -1:1, J, J);
D = spdiags([hh/tau],0,J,J); 
A = D + omega*L_h;
B = D - (1 - omega)*L_h;
A(1,1) = 1; A(1,2) = 0;		% Correction for Dirichlet 
B(1,1) = 0; B(1,2) = 0;		%  boundary condition
[L,U] = lu(A);			% Save LU decomposition
vo = exact_solution(0,x',mu,alpha,beta,u);	% Initial condition
vn = vo;					% Preallocation 

	% Time stepping with omega-scheme
tic
for n = 1:N
  t = n*tau;
  rhs = hh.*(omega*right_side(t,x',mu,beta,u) + ...
       (1-omega)*right_side(t-tau,x',mu,beta,u));
  rhs(1) = exact_solution(t,0,mu,alpha,beta,u);
  if inhom == 1			% Inhomogeneous Neumann condition
    rhs(J) = rhs(J) + mu*(omega*fR(t,mu,alpha,beta,u) +...
  		(1-omega)*fR(t-tau,mu,alpha,beta,u));
  end
  vn = U\(L\(B*vo + rhs));  vo = vn;
end
toc
		% Plot numerical solution at t = T
figure(1), clf, hold on
plot(x,vn,'*','markersize',13)
plot(x',exact_solution(t,x',mu,alpha,beta,u))
ha = findobj; set(ha(3),'FontSize',18)
d = 2*mu*tau/h^2;		% Diffusion number
c = u*tau/h;			% CFL number
p = u*h/mu;			% Mesh Peclet number 
title(['d=',num2str(d,2),' c=',num2str(c,2),' p=',num2str(p,2),'   ',...
  num2str(J-1),' cells  \omega=',num2str(omega,2),' T=',num2str(T,2)],...
  'fontsize',20)
box on
